/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import org.eclipse.jface.viewers.BaseLabelProvider;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.ui.ToolingKernelUIActivator;
import org.fortiss.tooling.kernel.ui.service.IMarkerService;
import org.fortiss.tooling.kernel.ui.util.ConstraintsUtils;
import org.fortiss.tooling.kernel.ui.util.ConstraintsUtils.ClassifiedStatuses;

/**
 * Label decorator for the {@link IMarkerService} decorations: errors, warnings, etc.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 22615D52E9BB387F51B1C3BE8A43207F
 */
public final class ConstraintLabelDecorator extends BaseLabelProvider implements
		ILightweightLabelDecorator {

	/** {@inheritDoc} */
	@Override
	public void decorate(Object element, IDecoration decoration) {
		if(element instanceof IConstrained) {
			IConstrained constrained = (IConstrained)element;
			ClassifiedStatuses classifiedStatuses =
					ConstraintsUtils.getClassifiedStatuses(constrained);
			if(!classifiedStatuses.failedStatuses.isEmpty()) {
				decoration.addOverlay(ToolingKernelUIActivator
						.getImageDescriptor("icons/constraintFailOverlay.png"));
			} else if(!classifiedStatuses.errorStatuses.isEmpty()) {
				decoration.addOverlay(ToolingKernelUIActivator
						.getImageDescriptor("icons/constraintErrorOverlay.png"));
			} else if(!classifiedStatuses.outdatedStatuses.isEmpty()) {
				decoration.addOverlay(ToolingKernelUIActivator
						.getImageDescriptor("icons/constraintOutdatedOverlay.png"));
			} else if(!classifiedStatuses.failedStatuses.isEmpty()) {
				decoration.addOverlay(null);
			}
		}
	}
}
