/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import java.util.ArrayList;
import java.util.List;

import org.fortiss.tooling.kernel.model.constraints.ConstraintsFactory;
import org.fortiss.tooling.kernel.model.constraints.ErrorVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.FailVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.OutdatedVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.SuccessVerificationStatus;

/**
 * Utility functions for constraints.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 6D5AD8843A4CA208840DD7AA8E3968C2
 */
public class ConstraintsUtils {
	/**
	 * @param constraint
	 * @return An "outdated" verification status for <code>constraint</code>.
	 */
	public static OutdatedVerificationStatus
			createOutdatedVerificationStatus(IConstraint constraint) {
		OutdatedVerificationStatus status =
				ConstraintsFactory.eINSTANCE.createOutdatedVerificationStatus();
		// The order if these two lines matter: otherwise we might get to some inconsistent state in
		// case some GUI element (like a table summarizing the status of all constraints) listens to
		// changes to the constraint.
		status.setConstraint(constraint);
		constraint.setVerificationStatus(status);
		return status;
	}

	/** Gathers some statuses by verification status. */
	public static class ClassifiedStatuses {

		/** Failed constraints. */
		public List<FailVerificationStatus> failedStatuses;

		/** Error statuses (constraint could not be checked). */
		public List<ErrorVerificationStatus> errorStatuses;

		/** Outdated statuses. */
		public List<OutdatedVerificationStatus> outdatedStatuses;

		/** Successful constraints. */
		public List<SuccessVerificationStatus> successStatuses;

		/** Constructor. */
		public ClassifiedStatuses() {
			failedStatuses = new ArrayList<FailVerificationStatus>();
			errorStatuses = new ArrayList<ErrorVerificationStatus>();
			outdatedStatuses = new ArrayList<OutdatedVerificationStatus>();
			successStatuses = new ArrayList<SuccessVerificationStatus>();
		}
	}

	/**
	 * @param constrained
	 * @return {@link ClassifiedStatuses} corresponding to <code>constrained</code>.
	 */
	public static ClassifiedStatuses getClassifiedStatuses(IConstrained constrained) {
		ClassifiedStatuses res = new ClassifiedStatuses();
		for(IConstraint constraint : constrained.getConstraints()) {
			IConstraintVerificationStatus status = constraint.getVerificationStatus();
			if(status instanceof FailVerificationStatus) {
				res.failedStatuses.add((FailVerificationStatus)status);
			}
			if(status instanceof ErrorVerificationStatus) {
				res.errorStatuses.add((ErrorVerificationStatus)status);
			}
			if(status instanceof OutdatedVerificationStatus) {
				res.outdatedStatuses.add((OutdatedVerificationStatus)status);
			}
			if(status instanceof SuccessVerificationStatus) {
				res.successStatuses.add((SuccessVerificationStatus)status);
			}
		}
		return res;
	}
}
